@@ -1,17 +1,17 @@
-eblupRY <- function(formula, D, T, vardir, method = c("REML", "ML"),
+eblupRY <- function(formula, D, TI, vardir, method = c("REML", "ML"),
          MAXITER = 1000, PRECISION = .1e-05, data, max.rho = .98, 
          dampening = 0.9, ...) { 
          
     if (length(method) > 1) method <- method[1]
     if (!method %in% c("REML", "ML", "MLE")) 
         stop(" method=\"", method, "\" must be \"REML\", or \"ML\"")
-    if(class(formula) == "list") {
+    if(inherits(formula, "list")) {
         NV <- length(formula)
         if(NV == 1) formula <- formula[[1]]
     } else { 
         NV <- 1
     }
-    if (class(vardir) == "list") {
+    if (inherits(vardir, "list")) {
         if (missing(D)) { 
             D <- length(vardir) 
         } else {
@@ -19,23 +19,23 @@ eblupRY <- function(formula, D, T, vardir, method = c("REML", "ML"),
                 stop("the length of 'vardir' must agree with D")
         }
         lapply(vardir, FUN = function(x) { 
-            if (!all(dim(x) == c(NV*T, NV*T))) 
+            if (!all(dim(x) == c(NV*TI, NV*TI))) 
                 stop(paste("each element of 'vardir' must be a square matrix",
-                           "with 'NV*T' rows"))
+                           "with 'NV*TI' rows"))
         })
-        vardir.temp <- matrix(0, nrow=D*NV*T, ncol=D*NV*T)
+        vardir.temp <- matrix(0, nrow=D*NV*TI, ncol=D*NV*TI)
         for (d in 1:D) {
-            vardir.temp[((d-1)*NV*T+1):(d*NV*T), ((d-1)*NV*T+1):(d*NV*T)] <-
+            vardir.temp[((d-1)*NV*TI+1):(d*NV*TI), ((d-1)*NV*TI+1):(d*NV*TI)] <-
                 vardir[[d]]
         }
         vardir <- vardir.temp
     } else {
         if (missing(D)) stop("'D' must be specified")
         if (!is.matrix(vardir)) vardir <- as.matrix(vardir)
-        if (class(vardir) != "matrix") 
+        if (!inherits(vardir, "matrix")) 
             stop("'vardir' must be a matrix or a list of matrices")
-        if (!all(dim(vardir) == c(D*NV*T, D*NV*T)))
-            stop("'vardir' must be a square matrix with 'D*NV*T' rows")
+        if (!all(dim(vardir) == c(D*NV*TI, D*NV*TI)))
+            stop("'vardir' must be a square matrix with 'D*NV*TI' rows")
     }
     if (NV == 1) {
         if (!missing(data)) {
@@ -46,10 +46,10 @@ eblupRY <- function(formula, D, T, vardir, method = c("REML", "ML"),
         X <- model.matrix(formula, data=formuladata)
         ncolx <- dim(X)[2]                                  # ncolx added
         nformula <- nrow(X)
-        if(nformula != D*T) 
-            stop("length of variables must be D * T")
+        if(nformula != D*TI) 
+            stop("length of variables must be D * TI")
         y <- formuladata[, 1]
-        result <- dynRYfit(y, X, M=D, T=T, NV=1, vcov_e=vardir,
+        result <- dynRYfit(y, X, M=D, TI=TI, NV=1, vcov_e=vardir,
                               maxiter=MAXITER, iter.tol=PRECISION, 
                               ncolx=ncolx, method=method, max.rho=max.rho, 
                               dampening=dampening, model="RY", ...)
@@ -73,8 +73,8 @@ eblupRY <- function(formula, D, T, vardir, method = c("REML", "ML"),
             ncolx[nv] <- dim(X1)[2]
             X.names <- append(X.names, paste0(attr(X1,"dimnames")[[2]], ".", nv))
         }
-        y <- rep(0, D*T*NV)
-        X <- matrix(0, nrow=D*T*NV, ncol=sum(ncolx))
+        y <- rep(0, D*TI*NV)
+        X <- matrix(0, nrow=D*TI*NV, ncol=sum(ncolx))
         attr(X, "dimnames")[[2]] <- X.names
         cstart <- 1
         for (nv in 1:NV) {
@@ -82,17 +82,17 @@ eblupRY <- function(formula, D, T, vardir, method = c("REML", "ML"),
             y1 <- formuladata[[nv]][, 1]
             cend <- cstart + ncol(X1) - 1
             nformula <- nrow(X1)
-            if(nformula != D*T) 
-                stop("length of variables must be D * T")
+            if(nformula != D*TI) 
+                stop("length of variables must be D * TI")
             for (m in 1:D) {
-                X[((m-1)*T*NV+(nv-1)*T+1):((m-1)*T*NV+nv*T), cstart:cend] <- 
-                    X1[((m-1)*T+1):(m*T), ]
-                y[((m-1)*T*NV+(nv-1)*T+1):((m-1)*T*NV+nv*T)] <-      
-                    y1[((m-1)*T+1):(m*T)] 
+                X[((m-1)*TI*NV+(nv-1)*TI+1):((m-1)*TI*NV+nv*TI), cstart:cend] <- 
+                    X1[((m-1)*TI+1):(m*TI), ]
+                y[((m-1)*TI*NV+(nv-1)*TI+1):((m-1)*TI*NV+nv*TI)] <-      
+                    y1[((m-1)*TI+1):(m*TI)] 
             }
             cstart <- cend + 1
         }
-        result <- dynRYfit(y, X, M=D, T=T, NV=NV, vcov_e=vardir,
+        result <- dynRYfit(y, X, M=D, TI=TI, NV=NV, vcov_e=vardir,
                               maxiter=MAXITER, iter.tol = PRECISION, 
                               ncolx=ncolx, method=method, max.rho=max.rho, 
                               dampening=dampening, model="RY", ...) 
